# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

"""
DB-API implementation based on pyhive
"""

import base64
import ssl

from pyhive import hive
from thrift.transport import THttpClient

# PEP 249 module globals
apilevel = hive.apilevel
threadsafety = hive.threadsafety
paramstyle = hive.paramstyle


def connect(
    database: str,
    http_path: str,
    username: str,
    password: str,
    host: str,
    port: int = 443,
    ssl_cert: str = "none",
    **kwargs
) -> hive.Connection:
    """
    Constructor for creating a pyhive DB-API connection to the SeqsLab cluster.
    :param database:
    :param http_path: a seqslab job run ID
    :param username:
    :param password:
    :param ssl_cert: "none", "optional", or "required"
    :param host:
    :param port:
    :returns: a pyhive :py:class:`Connection` object.
    """
    if ssl_cert:
        ssl_context = ssl.create_default_context()
        ssl_context.check_hostname = False
        ssl_context.verify_mode = hive.ssl_cert_parameter_map[ssl_cert]
    else:
        raise ValueError("Missing SSL certificate argument.")

    if username is not None and password is not None:
        auth = base64.standard_b64encode(f"{username}:{password}".encode()).decode("UTF-8")
    else:
        raise ValueError("Missing either username or password argument.")

    if http_path is not None:
        uri = f"https://{host}:{port}/{http_path}"
    else:
        raise ValueError("Missing http_path argument.")

    thrift_transport = THttpClient.THttpClient(
        uri_or_host=uri,
        ssl_context=ssl_context,
    )
    thrift_transport.setCustomHeaders({"Authorization": f"Basic {auth}"})

    return hive.Connection(database=database, thrift_transport=thrift_transport, **kwargs)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

"""
SQLAlchemy dialect for seqslab.hive
"""

import re

from seqslab import hive
from pyhive.sqlalchemy_hive import HiveDialect, _type_map
from sqlalchemy import types
from sqlalchemy import util


class SeqsLabHiveDialect(HiveDialect):
    """
    seqslab+hive://<username>:<password>@<host>:<port>/<database>?
    http_path=<job_run_id>&ssl_cert=<none|optional|required>
    """
    name = "seqslab"
    scheme = "https"
    driver = "hive"

    @classmethod
    def dbapi(cls):
        return hive

    def create_connect_args(self, url):
        kwargs = {
            "host": url.host,
            "port": url.port or 443,
            "username": url.username,
            "password": url.password,
            "database": re.sub(r'[^0-9a-zA-Z]', '_', url.database).lower() or "default",
        }

        if url.query is not None and "http_path" in url.query:
            kwargs["http_path"] = url.query["http_path"]

        kwargs.update(url.query)
        return [], kwargs

    def get_table_names(self, connection, schema=None, **kw):
        query = 'SHOW TABLES'
        if schema:
            query += ' IN ' + self.identifier_preparer.quote_identifier(schema)
        return [row[1] for row in connection.execute(query)]

    def get_columns(self, connection, table_name, schema=None, **kw):
        rows = self._get_table_columns(connection, table_name, schema)
        # Strip whitespace
        rows = [[col.strip() if col else None for col in row] for row in rows]
        # Filter out empty rows and comment
        rows = [row for row in rows if row[0] and row[0] != '# col_name']
        result = []
        for (col_name, col_type, _comment) in rows:
            if col_type is None:
                continue
            if col_name == '# Partition Information':
                break
            # Take out the more detailed type information
            # e.g. 'map<int,int>' -> 'map'
            #      'decimal(10,1)' -> decimal
            col_type = re.search(r'^\w+', col_type).group(0)
            try:
                coltype = _type_map[col_type]
            except KeyError:
                util.warn("Did not recognize type '%s' of column '%s'" % (col_type, col_name))
                coltype = types.NullType

            result.append({
                'name': col_name,
                'type': coltype,
                'nullable': True,
                'default': None,
            })
        return result



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

"""
Superset db engine specs for SeqsLab
"""

from superset.db_engine_specs.hive import HiveEngineSpec


class SeqsLabHiveEngineSpec(HiveEngineSpec):
    engine_name = "Atgenomix SeqsLab Interactive Job"

    engine = "seqslab"
    drivers = {"hive": "Hive driver for interactive job"}
    default_driver = "hive"

    _show_functions_column = "function"

    _time_grain_expressions = {
        None: "{col}",
        "PT1S": "date_trunc('second', {col})",
        "PT1M": "date_trunc('minute', {col})",
        "PT1H": "date_trunc('hour', {col})",
        "P1D": "date_trunc('day', {col})",
        "P1W": "date_trunc('week', {col})",
        "P1M": "date_trunc('month', {col})",
        "P3M": "date_trunc('quarter', {col})",
        "P1Y": "date_trunc('year', {col})",
        "P1W/1970-01-03T00:00:00Z": "date_trunc('week', {col} + interval '1 day') + interval '5 days'",
        "1969-12-28T00:00:00Z/P1W": "date_trunc('week', {col} + interval '1 day') - interval '1 day'",
    }



#!/usr/bin/env python3

import os
from typing import Union, Any

from setuptools import setup

import seqslab

setup_file_loc: Union[Union[str, bytes], Any] = os.path.abspath(
    os.path.dirname(__file__))
# allow setup.py to be run from any path
os.chdir(setup_file_loc)

extras_require = {}


def get_requirement():
    requirements = [  # dependency list
        'pip>=22.0.4'
    ]
    with open(os.path.join(setup_file_loc, 'requirements.txt'), 'r') as f:
        ori_req = f.read().splitlines()
    requirements.extend(ori_req)
    return requirements


def readme():
    path = os.path.join(setup_file_loc, 'README.md')
    with open(path, "r", encoding="utf-8") as fh:
        return fh.read()


setup(
    name="seqslab-connector",
    version=seqslab.__version__,
    description="Atgenomix SeqsLab Connector for Python",
    long_description=readme(),
    long_description_content_type="text/markdown",
    url="https://github.com/atgenomix/seqslab-connector",
    author="Allen Chang",
    author_email="allen.chang@atgenomix.com",
    license="Apache License, Version 2.0",
    packages=["seqslab", "seqslab.sqlalchemy", "seqslab.superset"],
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Topic :: Database :: Front-Ends",
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10'
    ],
    data_files=[('requirements', ['requirements.txt'])],
    install_requires=get_requirement(),
    extras_require={
        "sqlalchemy": ["sqlalchemy>=1.3.0"],
        "superset": ["superset>=2.0.1"],
    },
    tests_require=[],
    cmdclass={},
    package_data={
        "": ["*.rst"],
    },
    entry_points={
        "sqlalchemy.dialects": [
            "seqslab.hive = seqslab.sqlalchemy.hive:SeqsLabHiveDialect",
        ],
        "superset.db_engine_specs": [
            "seqslab = seqslab.superset.seqslab:SeqsLabHiveEngineSpec",
        ],
    }
)
